﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Olap.FlatData;
using Infragistics.Olap;
using Infragistics.Olap.Data;
using Infragistics.Controls.Grids;
using System.Threading;
using System.ComponentModel;

namespace PivotGridTest
{

    public class XAMPivotFilterHelper<T>
    {
        public FlatDataSource FlatDataSource { get; set; }
        public List<T> Data { get; set; }
        private ManualResetEvent _mre;

        public void SetFilter(IFilterMember filter, string value, bool set)
        {
            _mre = new ManualResetEvent(false);

            if (filter.Caption == value)
            {
                filter.IsSelected = set;
            }

            /* expand the filter if closed */
            if (!filter.IsExpanded)
            {
                filter.IsExpanded = true;
                _mre.WaitOne();
            }

            foreach (var child in filter.FilterMembers)
            {
                SetFilter(child, value, set);
            }
        }

        public IFilterViewModel GetFilterViewModel(string caption)
        {
            foreach (var filter in FlatDataSource.Filters)
            {
                if (filter.Caption == caption)
                {
                    return (IFilterViewModel)filter;
                }
            }

            foreach (var row in FlatDataSource.Rows)
            {
                if (row.Caption == caption)
                {
                    return (IFilterViewModel)row;
                }
            }

            foreach (var column in FlatDataSource.Columns)
            {
                if (column.Caption == caption)
                {
                    return (IFilterViewModel)column;
                }
            }

            return null;
        }


        public void ExpandHierarchy(IFilterMember filter, bool value)
        {
            _mre = new ManualResetEvent(false);

            if (filter.IsExpanded != value)
            {
                filter.IsExpanded = value;
            }

            if (filter.FilterMembers == null)
            {
                _mre.WaitOne();
            }

            foreach (var child in filter.FilterMembers)
            {
                ExpandHierarchy(child, value);
            }
        }


        public void FilterOption(string tag, string value, bool set = true)
        {
            IFilterViewModel fvm = GetFilterViewModel(tag);

            if (fvm == null)
                return;

            fvm.LoadFilterMembersCompleted += (s, e) => _mre.Set();

            if (fvm.FilterMembers != null)
            {
                foreach (var filter in fvm.FilterMembers)
                {
                    SetFilter(filter, value, set);
                }
            }

            fvm.CommitChanges();
        }


        public void FilterOption(IFilterViewModel fvm, string value, bool set = true)
        {
            fvm.LoadFilterMembersCompleted += (s, e) => _mre.Set();

            if (fvm.FilterMembers != null)
            {
                foreach (var filter in fvm.FilterMembers)
                {
                    SetFilter(filter, value, set);
                }
            }

            fvm.CommitChanges();
        }


        public void OnResultChanged(object sender, AsyncCompletedEventArgs e)
        {
            Data = new List<T>();

            List<int> indices = FlatDataSource.GetCellItemsIndexes(
                FlatDataSource.Result.Cells[FlatDataSource.Result.Cells.GetUpperBound(0), FlatDataSource.Result.Cells.GetUpperBound(1)]);

            foreach (int index in indices)
                Data.Add(((T)FlatDataSource.GetRecord(index)));
        }

        public void HierarchyChanged(object sender, AsyncCompletedEventArgs e)
        {
            DataSourceBase dataSourceBase = (DataSourceBase)sender;
            dataSourceBase.ResultChanged -= HierarchyChanged;

            dataSourceBase.HierarchyExpansionMode = HierarchyExpansionMode.DrillDown;
            dataSourceBase.UpdateVisualMembers(Infragistics.Olap.ExecutionContext.Rows);
            dataSourceBase.UpdateVisualMembers(Infragistics.Olap.ExecutionContext.Columns);
            dataSourceBase.HierarchyExpansionMode = HierarchyExpansionMode.SingleLevelOnly;

            dataSourceBase.RefreshGrid();
        }
    }
    
}
